function [Uf] = U_f(f,n,m,arg_f)

// Uf = U_f(f,n,m,args_f)
//  Construye la transformacion unitaria U_f, que implementa la funcion booleana clasica
//  f (nombre de la funcion). Las entradas y salidas de f se representan en
//  decimal. Se puede pasar un argumento adicional a f en arg_f.
//  El dominio de f tiene cardinal 2^n y el codominio 2^m.

//  la construccion de Uf se realiza de la siguiente manera:
//   |x,y> -> |x,y xor f(x)>
//   siendo y un elemento de la base canonica del codominio, de esa forma,
//   al utilizar estados puros, la U_f no es otra cosa que una permutacion
//   adecuada de I

hay_arg = 0;
if (argn(2) == 4), hay_arg = 1; end

Uf = zeros(2^(m+n));

ind = 1;
for i=0:2^n-1
    for j=0:2^m-1
        if (hay_arg)                       // computo el valor de fx (en decimal)
            fx = feval(i,arg_f,f);         //
        else                               //
            fx = feval(i,f);               //
        end
        bfx = modulo(fx+j,2^m);         // computo j xor fx (en decimal)
        eb = bfx+i*2^m;                 // encuentro el elemento de la base que debo permutar
                                        // (a bfx le sumo el valor de x
                                        // desplazado m bits a la derecha)
        Uf(eb+1,ind) = 1;
        ind = ind + 1;
    end
end

endfunction
